# Atlernatives to `Functor` trait

## `_`-`FnOnce` category functors (current)

Copied for reference. All following examples are in the same `Functor`-`Applicative`-`Monad` format
without extra (sub)traits like `WeakFunctor`, `Pure`, `ApplicativeLA2`, etc. .

```rust
trait Functor<'a>: 'a {
    type F<A: 'a>: 'a;

    fn fmap<A: 'a, B: 'a>(
        fa: Self::F<A>, f: impl 'a + FnOnce(A) -> B,
    ) -> Self::F<B>;
}

fn fmap<'a, T: Functor<'a>, A: 'a, B: 'a>(
    f: impl 'a + FnOnce(A) -> B,
) -> impl FnOnce(<T as Functor<'a>>::F<A>) -> <T as Functor<'a>>::F<B> {
    |fa| T::fmap(fa, f)
}
```

## `Clone`-`Fn` category functors

This is probably the closest representation to what Haskell views as a category of its types.

```rust
trait Functor<'a>: 'a + Clone {
    type F<A: 'a + Clone>: 'a + Clone;

    fn fmap<A: 'a + Clone, B: 'a + Clone>(
        fa: Self::F<A>, f: impl 'a + Clone + Fn(A) -> B,
    ) -> Self::F<B>;
}

fn fmap<'a, T: Functor<'a>, A: 'a + Clone, B: 'a + Clone>(
    f: impl 'a + Clone + Fn(A) -> B,
) -> impl 'a + Clone + Fn(<T as Functor<'a>>::F<A>) -> <T as Functor<'a>>::F<B> {
    move |fa| T::fmap(fa, f.clone())
}
```

## `Clone`-`FnMut` category functors

We view use of `FnMut` for category's morphisms as somewhat controversial[^e].

* Use of direct/indirect mutable references is, arguably, counter-functional[^e].
* `Clone+FnMut` is, generally, nonsensical[^e].
* Due to that, morphisms category isn't a subcategory, so can't be wrapped.
* Not being to wrap morphisms makes implementation of one specific `Applicative` method,
sequential application (`<*>` in Haskell, `seq` in RADN) *difficult*.

```rust
trait Functor<'a>: 'a + Clone {
    type F<A: 'a + Clone>: 'a + Clone;

    fn fmap<A: 'a + Clone, B: 'a + Clone>(
        fa: Self::F<A>, f: impl 'a + FnMut(A) -> B,
    ) -> Self::F<B>;
}
```

[^e]: elaborate?

## `Copy`-`Fn` category functors

```rust
trait Functor<'a>: 'a + Copy {
    type F<A: 'a + Copy>: 'a + Copy;

    fn fmap<A: 'a + Copy, B: 'a + Copy>(
        fa: Self::F<A>, f: impl 'a + Copy + Fn(A) -> B,
    ) -> Self::F<B>;
}

fn fmap<'a, T: Functor<'a>, A: 'a + Copy, B: 'a + Copy>(
    f: impl 'a + Copy + Fn(A) -> B,
) -> impl 'a + Copy + Fn(<T as Functor<'a>>::F<A>) -> <T as Functor<'a>>::F<B> {
    move |fa| T::fmap(fa, f)
}
```
